const path = require("path");
const webpack = require("webpack");
const BundleAnalyzerPlugin = require("webpack-bundle-analyzer")
  .BundleAnalyzerPlugin;
const ManifestPlugin = require("webpack-manifest-plugin");
const utils = require("../utils");

const config = {
  mode: "production",

  bail: true,

  resolve: {
    extensions: [".ts", ".tsx", ".js", ".jsx"],
    modules: [path.resolve(__dirname), "node_modules", "app", "app/redux"]
  },

  entry: {
    app: "./src/client.tsx"
  },

  output: {
    path: path.resolve("./build/public"),
    publicPath: "/public/",
    filename: "js/[name].[chunkhash].js"
  },

  module: {
    rules: [
      // {
      //   enforce: 'pre',
      //   test: /\.tsx?$/,
      //   loader: 'eslint-loader'
      // },
      {
        test: /\.tsx?$/,
        loader: "ts-loader",
        exclude: /node_modules/
      },
      {
        test: /\.eot(\?.*)?$/,
        loader: "file-loader?name=fonts/[hash].[ext]"
      },
      {
        test: /\.(woff|woff2)(\?.*)?$/,
        loader: "file-loader?name=fonts/[hash].[ext]"
      },
      {
        test: /\.ttf(\?.*)?$/,
        loader:
          "url-loader?limit=10000&mimetype=application/octet-stream&name=fonts/[hash].[ext]"
      },
      {
        test: /\.svg(\?.*)?$/,
        loader:
          "url-loader?limit=10000&mimetype=image/svg+xml&name=fonts/[hash].[ext]"
      },
      {
        test: /\.(jpe?g|png|gif)$/i,
        loader: "url-loader?limit=1000&name=images/[hash].[ext]"
      }
    ]
  },

  plugins: [
    new webpack.LoaderOptionsPlugin({
      debug: true,
      options: {
        tslint: {
          failOnHint: true
        }
      }
    }),
    new ManifestPlugin({
      fileName: "../manifest.json"
    }),
    new webpack.DefinePlugin({
      "process.env": {
        BROWSER: JSON.stringify(true),
        NODE_ENV: JSON.stringify("production")
      }
    })
    // Uncomment this to analyze bundle
    // new BundleAnalyzerPlugin()
  ],

  optimization: {
    splitChunks: {
      chunks: "all"
    }
  }
};

utils.copySyncIfDoesntExist("./config/main.js", "./config/main.local.js");
utils.createIfDoesntExist("./build");
utils.createIfDoesntExist("./build/public");
utils.copySync("./src/favicon.ico", "./build/public/favicon.ico", true);

module.exports = config;
