const path = require("path");
const fs = require("fs");
const utils = require("../utils");

const nodeModules = {};
fs.readdirSync("node_modules")
  .filter(function(x) {
    return [".bin"].indexOf(x) === -1;
  })
  .forEach(function(mod) {
    nodeModules[mod] = "commonjs " + mod;
  });

const config = {
  mode: process.env.NODE_ENV === "production" ? "production" : "development",
  externals: nodeModules,
  target: "node",

  resolve: {
    extensions: [".ts", ".tsx", ".js", ".jsx"],
    modules: [path.resolve(__dirname), "node_modules", "app", "app/redux"]
  },

  entry: "./src/server.tsx",

  output: {
    path: path.resolve("./build/public"),
    filename: "../server.js",
    publicPath: "/public/",
    libraryTarget: "commonjs2"
  },

  module: {
    rules: [
      {
        test: /\.tsx?$/,
        loader: "ts-loader",
        exclude: /node_modules/
      },
      {
        test: /\.eot(\?.*)?$/,
        loader: "file-loader?name=fonts/[hash].[ext]"
      },
      {
        test: /\.(woff|woff2)(\?.*)?$/,
        loader: "file-loader?name=fonts/[hash].[ext]"
      },
      {
        test: /\.ttf(\?.*)?$/,
        loader:
          "url-loader?limit=10000&mimetype=application/octet-stream&name=fonts/[hash].[ext]"
      },
      {
        test: /\.svg(\?.*)?$/,
        loader:
          "url-loader?limit=10000&mimetype=image/svg+xml&name=fonts/[hash].[ext]"
      },
      {
        test: /\.(jpe?g|png|gif)$/i,
        loader: "url-loader?limit=10000&name=images/[hash].[ext]"
      }
    ]
  },

  plugins: [],

  node: {
    console: false,
    global: false,
    process: false,
    Buffer: false,
    __filename: false,
    __dirname: false
  }
};

utils.copySyncIfDoesntExist("./config/main.js", "./config/main.local.js");
utils.createIfDoesntExist("./build");
utils.createIfDoesntExist("./build/public");
utils.copySync("./src/favicon.ico", "./build/public/favicon.ico", true);

module.exports = config;
